/*
 * ti_file - a class used to handle files for ti calculators
 *
 * Copyright (C) 2010 Michael Jordan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef TI_FILE_H_
#define TI_FILE_H_

#include <asm-generic/errno-base.h>
#include <errno.h>
#include <sysexits.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>
#include <iostream>
using namespace std;

// the ti_file class handles var files contained on disk (opening, parsing, and
// returning the actual var data).

class ti_file {
public:

	// ti_file(string filename) is passed a filename string and proceeds to open
	// and parse the file into its individual items (vars) from within the file.
	// The header bytes are first verified, then parsing of the various sections
	// of the file occurs into memory after which time, if successful, the data
	// and its properties may be accessed via the class's member functions.  Only
	// after successful initialization will any of the access functions return
	// valid values - else errors are returned

	ti_file(string filename);

	// ~ti_file() is responsible for destroying all of the memory allocated to
	// hold the vars from string filename in memory

	~ti_file();

	// is_flash() returns true if string filename is a flash file

	bool is_flash();

	// is_var() returns true if string filename is a var file

	bool is_var();

	// num_entries() returns the number of entries found in string filename

	int num_entries();

	// data_size(int ent_no) returns the size of the data for an entry in
	// string filename - entries are zero based, so the first var found in the
	// file is 0, the second is 1, etc.

	int data_size(int ent_no);

	// cs_ok(int ent_no) returns true if the checksum on the var was validated
	// successfully

	bool cs_ok(int ent_no);

	// is_good() returns true if string filename was successfully opened and
	// parsed into memory

	bool is_good();

	// data(int ent_no, unsigned char *buf) returns the data for var entry ent_no
	// contained in string filename - the data is returned in buf, which must be
	// allocated and managed by the caller

	bool data(int ent_no, unsigned char *buf);

	// name(int ent_no) returns the name of var ent_no in string filename

	string name(int ent_no);

	// archived(int ent_no) returns true if the archived bit is set for var ent_no
	// in string filename

	bool archived(int ent_no);

	// locked(int ent_no) returns true if the locked bit is set for var ent_no
	// in string filename

	bool locked(int ent_no);

	// comment() returns the comment contained in a flash file for its sole var

	string comment();

	// date() returns the date contained in a flash file for its sole var

	string date();

	// type_id(int ent_no) returns the type id field for var ent_no in string
	// filename

	unsigned int type_id(int ent_no);

	// dev_type() returns the dev type entry contained in a flash file for
	// its sole var

	int dev_type();

	// data_type() returns the data type entry contained in a flash file for
	// its sole var

	int data_type();

	// hdw_id() returns the hdw id entry contained in a flash file for its sole
	// var

	int hdw_id();

	// folder(int ent_no) returns the folder name for var ent_no in string
	// filename

	string folder(int ent_no);

	// version() returns the version entry contained in a flash file for its sole
	// var

	string version();

	// typeid_to_string(int type_id) returns a string matching type_id as displayed
	// on the calculator's file listing (i.e. PRGM, FIG, etc.)

	string typeid_to_string(int type_id);

	// get_sys_directory(string dir, vector<string> &files) gets a listing of the
	// files in the current directory.  The application needs to pass an
	// instance of vector<string> in order for the values to be filled in and
	// returned to the application.  The application must then take care of
	// cleaning up after it is done with the vector.

	int get_sys_directory(string dir, vector<string> &files);

private:

	// is_valid(string filename) tests filename to determine:
	// 1.  filename is not a directory
	// 2.  filename does exist and it can be opened by this application

	bool is_valid(string filename);

	// is_inited tracks whether ti_file was able to successfully open and parse
	// filename.  Until is_inited is true, none of the access methods return
	// valid values.

	bool is_inited;

	// Upon successful initialization, head contains the id string of filename
	// (**TI89**, **TI92P*, etc.) as well as a void pointer to the initial
	// header in the file.  The pointer must be cast to the correct type based
	// on id_str to be able to access the subsequent properties.  entry will be
	// either a pointer to a flash_head object or a pointer to a var_head
	// object.

	class head {
	public:
		string id_str;
		void *entry;
	} hd;

	// flash_head stores the information contained in the header of a flash
	// file.  Flash files (of the proper TI breed) are single item files only.
	// Therefore when a flash file is initialized, head.entry will point to
	// a single instance of a flash_head object.  The data member is a pointer
	// to a buffer containing the actual data for the var.

	class flash_head {
	public:
		unsigned char version[2];
		unsigned char bcd_date[4];
		string name;
		int dev_type;
		int data_type;
		int hdw_id;
		int data_size;
		unsigned char *data;
	};

	// vf_entry contains the information from the header for each individual var
	// contained in the TI file.  The data member is a pointer to a buffer
	// containing the actual data for the var.

	class vf_entry {
	public:
		string parent_folder;
		int offset;
		string name;
		unsigned int type_id;
		bool archived;
		bool locked;
		bool cksum_ok;
		int data_len;
		int num_entries;
		unsigned char *data;
	};

	// var_head stores the first header information contained in a var file.  As
	// var files can contain more than one var per file, var_head.entries will
	// point to a list of one or more vf_entry objects.

	class var_head {
	public:
		string fld_default;
		string comment;
		int num_entries;
		vf_entry *entries;
	};

};

#endif /* TI_FILE_H_ */
